"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var utils = require("tns-core-modules/utils/utils");
var arcommon_1 = require("./arcommon");
var pixelsPerMeter = 500;
var alignCenter = function (node) {
    node.setLocalPosition(new com.google.ar.sceneform.math.Vector3(0, -node.getLocalScale().y / 2, 0));
};
var alignBottom = function (node) {
    node.setLocalPosition(new com.google.ar.sceneform.math.Vector3(0, 0, 0));
};
var ARVideo = (function (_super) {
    __extends(ARVideo, _super);
    function ARVideo() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ARVideo.prototype.isPlaying = function () {
        return this.mediaPlayer && this.mediaPlayer.isPlaying();
    };
    ARVideo.prototype.play = function () {
        if (this.mediaPlayer) {
            this.mediaPlayer.start();
        }
    };
    ARVideo.prototype.pause = function () {
        if (this.mediaPlayer) {
            this.mediaPlayer.pause();
        }
    };
    ARVideo.create = function (options, fragment) {
        var _this = this;
        return new Promise(function (resolve, reject) { return __awaiter(_this, void 0, void 0, function () {
            var node, videoNode, size, texture, mediaPlayer, loop, videoMat;
            return __generator(this, function (_a) {
                node = arcommon_1.ARCommonNode.createNode(options, fragment);
                videoNode = arcommon_1.ARCommonNode.createNode(options, fragment);
                videoNode.setParent(node);
                size = new com.google.ar.sceneform.math.Vector3(options.dimensions instanceof Object ? options.dimensions.x : options.dimensions || 0.96, options.dimensions instanceof Object ? options.dimensions.y : options.dimensions || 0.56, 1);
                videoNode.setLocalScale(size);
                alignCenter(videoNode);
                texture = new com.google.ar.sceneform.rendering.ExternalTexture();
                mediaPlayer = ARVideo.getPlayer(options);
                mediaPlayer.setSurface(texture.getSurface());
                mediaPlayer.setVideoScalingMode(android.media.MediaPlayer.VIDEO_SCALING_MODE_SCALE_TO_FIT_WITH_CROPPING);
                loop = options.loop !== false;
                if (loop) {
                    mediaPlayer.setLooping(true);
                }
                ARVideo.getModel().then(function (renderable) {
                    videoMat = renderable.getMaterial();
                    com.google.ar.sceneform.rendering.MaterialFactory.makeOpaqueWithColor(utils.ad.getApplicationContext(), new com.google.ar.sceneform.rendering.Color(android.graphics.Color.MAGENTA))
                        .thenAccept(new java.util.function.Consumer({
                        accept: function (material) {
                            renderable.setMaterial(material);
                            videoNode.setRenderable(renderable);
                            var arVideo = new ARVideo(options, node);
                            arVideo.mediaPlayer = mediaPlayer;
                            resolve(arVideo);
                        }
                    }))
                        .exceptionally(new java.util.function.Function({
                        apply: function (error) { return reject(error); }
                    }));
                    videoMat.setExternalTexture("videoTexture", texture);
                    videoMat.setBoolean("disableChromaKey", true);
                    mediaPlayer.setOnPreparedListener(new android.media.MediaPlayer.OnPreparedListener({
                        onPrepared: function (mp) {
                            var width = mp.getVideoWidth();
                            var height = mp.getVideoHeight();
                            if (!options.dimensions) {
                                videoNode.setLocalScale(new com.google.ar.sceneform.math.Vector3(width / pixelsPerMeter, height / pixelsPerMeter, 1));
                            }
                            alignCenter(videoNode);
                            console.log([height, width]);
                            mediaPlayer.start();
                            texture
                                .getSurfaceTexture()
                                .setOnFrameAvailableListener(new android.graphics.SurfaceTexture.OnFrameAvailableListener({
                                onFrameAvailable: function (surfaceTexture) {
                                    console.log('available');
                                    try {
                                        renderable.setMaterial(videoMat);
                                        texture.getSurfaceTexture().setOnFrameAvailableListener(null);
                                    }
                                    catch (e) {
                                        console.error(e);
                                    }
                                }
                            }));
                        }
                    }));
                    mediaPlayer.prepareAsync();
                }).catch(console.error);
                return [2];
            });
        }); });
    };
    ARVideo.getModel = function () {
        return new Promise(function (resolve, reject) {
            try {
                com.google.ar.sceneform.rendering.ModelRenderable.builder()
                    .setSource(utils.ad.getApplicationContext(), android.net.Uri.parse("video_chroma.sfb"))
                    .build()
                    .thenAccept(new java.util.function.Consumer({
                    accept: function (renderable) {
                        resolve(renderable);
                    }
                }))
                    .exceptionally(new java.util.function.Function({
                    apply: function (error) {
                        console.log("g");
                        reject(error);
                    }
                }));
            }
            catch (e) {
                reject(e);
            }
        });
    };
    ARVideo.getPlayer = function (options) {
        var video = options.video;
        var context = utils.ad.getApplicationContext();
        if (typeof video === "string") {
            try {
                var mediaPlayer = new android.media.MediaPlayer();
                if (video.indexOf("://") >= 0) {
                    mediaPlayer.setDataSource(context, android.net.Uri.parse(video));
                }
                else {
                    mediaPlayer.setDataSource(context.getAssets().openFd(video));
                }
                mediaPlayer.setOnErrorListener(new android.media.MediaPlayer.OnErrorListener({
                    onError: function (mp, what, extra) {
                        console.error("MediaPlayer Error " + what + " with " + video);
                        ([
                            [android.media.MediaPlayer.MEDIA_ERROR_IO, "MEDIA_ERROR_IO"],
                            [android.media.MediaPlayer.MEDIA_ERROR_MALFORMED, "MEDIA_ERROR_MALFORMED"],
                            [android.media.MediaPlayer.MEDIA_ERROR_NOT_VALID_FOR_PROGRESSIVE_PLAYBACK, "MEDIA_ERROR_NOT_VALID_FOR_PROGRESSIVE_PLAYBACK"],
                            [android.media.MediaPlayer.MEDIA_ERROR_SERVER_DIED, "MEDIA_ERROR_SERVER_DIED"],
                            [android.media.MediaPlayer.MEDIA_ERROR_TIMED_OUT, "MEDIA_ERROR_TIMED_OUT"],
                            [android.media.MediaPlayer.MEDIA_ERROR_UNKNOWN, "MEDIA_ERROR_UNKNOWN"],
                            [android.media.MediaPlayer.MEDIA_ERROR_UNSUPPORTED, "MEDIA_ERROR_UNSUPPORTED"]
                        ]).forEach(function (code) {
                            if (what === code[0]) {
                                console.log(code[1]);
                            }
                            if (extra === code[0]) {
                                console.log(code[1]);
                            }
                        });
                        return true;
                    }
                }));
                return mediaPlayer;
            }
            catch (e) {
                console.error(video);
                console.error(e);
            }
        }
        console.log('throw');
        throw 'Error';
    };
    return ARVideo;
}(arcommon_1.ARCommonNode));
exports.ARVideo = ARVideo;
